@@ -1,13 +1,13 @@
 #if !defined(lint) && !defined(__INSIGHT__)
-static char sos__rcsid[] = "errno.c,v 1.1.1.1 1995/06/16 17:42:41 seth Exp";
-static char sos__copyright[] = "Copyright (c) 1994, 1995 SOS Corporation";
+static char sos__rcsid[] = "$Id$";
+static char sos__copyright[] = "Copyright (c) 1994, 1995, 1996 SOS Corporation";
 static char sos__contact[] = "SOS Corporation <sos-info@soscorp.com> +1 800 SOS UNIX";
 #endif /* not lint */
 
 /*
  * ++Copyright Released Product++
  *
- * Copyright (c) 1994, 1995 Sources of Supply Corporation ("SOS").
+ * Copyright (c) 1994, 1995, 1996 Sources of Supply Corporation ("SOS").
  * All rights reserved.
  *
  * The SOS Released Product License Agreement specifies the terms and
@@ -26,9 +26,6 @@ static char sos__contact[] = "SOS Corporation <sos-info@soscorp.com> +1 800 SOS
 
 #include "sos.h"
 
-extern int errno;
-extern char *sys_errlist[];
-
 int sos_errno;			
 char *sos_errlist[] = 
 {
@@ -45,7 +42,7 @@ sos_perror(char *s)
   if ( sos_errno == SOS_ERRNO_SYS )
     fprintf(stderr, "%s: %s: %s\n", s, 
 	    sos_errlist[sos_errno],
-	    sys_errlist[sos_errno]);
+	    strerror(errno));
   else
     fprintf(stderr, "%s: %s\n", s, sos_errlist[sos_errno]);
 }
