@@ -1,13 +1,13 @@
 #if !defined(lint) && !defined(__INSIGHT__)
-static char sos__rcsid[] = "fqhname.c,v 1.1.1.1 1995/06/16 17:42:41 seth Exp";
-static char sos__copyright[] = "Copyright (c) 1994, 1995 SOS Corporation";
+static char sos__rcsid[] = "$Id$";
+static char sos__copyright[] = "Copyright (c) 1994, 1995, 1996 SOS Corporation";
 static char sos__contact[] = "SOS Corporation <sos-info@soscorp.com> +1 800 SOS UNIX";
 #endif /* not lint */
 
 /*
  * ++Copyright Released Product++
  *
- * Copyright (c) 1994, 1995 Sources of Supply Corporation ("SOS").
+ * Copyright (c) 1994, 1995, 1996 Sources of Supply Corporation ("SOS").
  * All rights reserved.
  *
  * The SOS Released Product License Agreement specifies the terms and
@@ -30,18 +30,24 @@ static char sos__contact[] = "SOS Corporation <sos-info@soscorp.com> +1 800 SOS
 
 #include "sos.h"
 
-/* Get the fully qualified host name */
+
+
+/*
+ * Get the fully qualified host name
+ */
 char *sos_fqhname(char *name,int namelen)
 {
+  SOS_ENTRY("sos_fqhname","sos_fqhname",NULL);
   char *fqname;
   static char statbuf[MAXHOSTNAMELEN+1];
 
   memset(statbuf,0,MAXHOSTNAMELEN+1);
 
   if (gethostname(statbuf,MAXHOSTNAMELEN) < 0)
-      {
-	return(NULL);
-      }
+    {
+      sos_error_printf("Could not get host name: %s\n",strerror(errno));
+      SOS_RETURN(NULL);
+    }
 
   fqname = sos_getnbyfoo(statbuf,NULL,0);
 
@@ -51,11 +57,14 @@ char *sos_fqhname(char *name,int namelen)
     }
 
   if (!name)
-    return(fqname);		/* Use static buffer */
+    SOS_RETURN(fqname);		/* Use static buffer */
 
   if (strlen(fqname) > namelen)
-    return (NULL);		/* Not long enough */
+    {
+      sos_error_printf("Could not fit FQ hostname into supplied buffer (%d > %d)\n",strlen(fqname),namelen);
+      SOS_RETURN(NULL);		/* Not long enough */
+    }
 
   strcpy(name,fqname);
-  return(name);
+  SOS_RETURN(name);
 }
