@@ -1,13 +1,13 @@
 #if !defined(lint) && !defined(__INSIGHT__)
-static char sos__rcsid[] = "rip.c,v 1.1.1.1 1995/06/16 17:42:43 seth Exp";
-static char sos__copyright[] = "Copyright (c) 1994, 1995 SOS Corporation";
+static char sos__rcsid[] = "$Id$";
+static char sos__copyright[] = "Copyright (c) 1994, 1995, 1996 SOS Corporation";
 static char sos__contact[] = "SOS Corporation <sos-info@soscorp.com> +1 800 SOS UNIX";
 #endif /* not lint */
 
 /*
  * ++Copyright Released Product++
  *
- * Copyright (c) 1994, 1995 Sources of Supply Corporation ("SOS").
+ * Copyright (c) 1994, 1995, 1996 Sources of Supply Corporation ("SOS").
  * All rights reserved.
  *
  * The SOS Released Product License Agreement specifies the terms and
@@ -35,29 +35,43 @@ static char sos__contact[] = "SOS Corporation <sos-info@soscorp.com> +1 800 SOS
 #define RIP_CHARS "\r\n"
 
 
-/* Turn first CR/LF found into a NULL */
+
+/*
+ * Turn first CR/LF found into a NULL
+ */
 char *sos_rip(char *x)
 {
+  SOS_ENTRY("sos_rip","sos_rip",NULL);
   char *ripped;
 
-  if (!x) return(x);
+  if (!x)
+    {
+      sos_error_printf("No deposit no return--please supply a buffer to rip\n");
+      SOS_RETURN(x);
+    }
 
   if (!(ripped = strpbrk(x,RIP_CHARS)))
-    return(x);
+    SOS_RETURN(x);
 
   *ripped = '\0';
-  return(x);
+  SOS_RETURN(x);
 }
 
 
+
 /*
  * Turn CRLF sequences into NULL or CRNULL
  */
 char *sos_fixcrnl(char *x, int stripnl)
 {
+  SOS_ENTRY("sos_rip","sos_fixcrnl",NULL);
   char *nl;
 
-  if (!x) return(x);
+  if (!x)
+    {
+      sos_error_printf("No deposit no return--please supply a buffer to rip\n");
+      SOS_RETURN(x);
+    }
 
   for(nl = strchr(x,'\n');nl;nl = strchr(nl+1,'\n'))
     {
@@ -73,5 +87,5 @@ char *sos_fixcrnl(char *x, int stripnl)
 	}
     }
 
-  return(x);
+  SOS_RETURN(x);
 }
